@@ -38,6 +38,7 @@
 #include <libweston/backend-drm.h>
 #include <libweston/pixel-formats.h>
 #include <libweston/linux-dmabuf.h>
+#include "shared/hash.h"
 #include "shared/helpers.h"
 #include "shared/weston-drm-fourcc.h"
 #include "drm-internal.h"
@@ -46,7 +47,12 @@
 static void
 drm_fb_destroy(struct drm_fb *fb)
 {
-	if (fb->fb_id != 0)
+	/* TODO: do not leak the fb during shutdown.
+	 * When we are shutting down, the CRTC may be scanning out our fb. If we
+	 * destroy it, the CRTC may turn off. So leak that on purpose. What we
+	 * need here is a libdrm function that we can use to tell DRM/KMS that
+	 * it is free to destroy our fb once it stops using it. */
+	if (fb->fb_id != 0 && !fb->backend->compositor->shutting_down)
 		drmModeRmFB(fb->fd, fb->fb_id);
 	free(fb);
 }
@@ -68,16 +74,152 @@ drm_fb_destroy_dumb(struct drm_fb *fb)
 	drm_fb_destroy(fb);
 }
 
+#ifdef BUILD_DRM_GBM
+static int gem_handle_get(struct drm_device *device, int handle)
+{
+	unsigned int *ref_count;
+
+	ref_count = hash_table_lookup(device->gem_handle_refcnt, handle);
+	if (!ref_count) {
+		ref_count = zalloc(sizeof(*ref_count));
+		hash_table_insert(device->gem_handle_refcnt, handle, ref_count);
+	}
+	(*ref_count)++;
+
+	return handle;
+}
+
+static void gem_handle_put(struct drm_device *device, int handle)
+{
+	unsigned int *ref_count;
+
+	if (handle == 0)
+		return;
+
+	ref_count = hash_table_lookup(device->gem_handle_refcnt, handle);
+	if (!ref_count) {
+		weston_log("failed to find GEM handle %d for device %s\n",
+			   handle, device->drm.filename);
+		return;
+	}
+	(*ref_count)--;
+
+	if (*ref_count == 0) {
+		hash_table_remove(device->gem_handle_refcnt, handle);
+		free(ref_count);
+		drmCloseBufferHandle(device->drm.fd, handle);
+	}
+}
+
 static int
-drm_fb_addfb(struct drm_backend *b, struct drm_fb *fb)
+drm_fb_import_plane(struct drm_device *device, struct drm_fb *fb, int plane)
+{
+	int bo_fd;
+	uint32_t handle;
+	int ret;
+
+	bo_fd = gbm_bo_get_fd_for_plane(fb->bo, plane);
+	if (bo_fd < 0)
+		return bo_fd;
+
+	/*
+	 * drmPrimeFDToHandle is dangerous, because the GEM handles are
+	 * not reference counted by the kernel and user space needs a
+	 * single reference counting implementation to avoid double
+	 * closing of GEM handles.
+	 *
+	 * It is not desirable to use a GBM device here, because this
+	 * requires a GBM device implementation, which might not be
+	 * available for simple or custom DRM devices that only support
+	 * scanout and no rendering.
+	 *
+	 * We are only importing the buffers from the render device to
+	 * the scanout device if the devices are distinct, since
+	 * otherwise no import is necessary. Therefore, we are the only
+	 * instance using the handles and we can implement reference
+	 * counting for the handles per device. See gem_handle_get and
+	 * gem_handle_put for the implementation.
+	 */
+	ret = drmPrimeFDToHandle(fb->fd, bo_fd, &handle);
+	if (ret)
+		goto out;
+
+	fb->handles[plane] = gem_handle_get(device, handle);
+
+out:
+	close(bo_fd);
+	return ret;
+}
+#endif
+
+/*
+ * If the fb is using a GBM surface, there is a possibility that the GBM
+ * surface has been created on a different device than the device which
+ * should be used for the fb. We have to import the fd of the GBM bo
+ * into the scanout device.
+ */
+static int
+drm_fb_maybe_import(struct drm_device *device, struct drm_fb *fb)
+{
+#ifndef BUILD_DRM_GBM
+	/*
+	 * Without GBM support, the fb is always allocated on the scanout device
+	 * and import is never necessary.
+	 */
+	return 0;
+#else
+	struct gbm_device *gbm_device;
+	int ret = 0;
+	int plane;
+
+	/* No import possible, if there is no gbm bo */
+	if (!fb->bo)
+		return 0;
+
+	/* No import necessary, if the gbm bo and the fb use the same device */
+	gbm_device = gbm_bo_get_device(fb->bo);
+	if (gbm_device_get_fd(gbm_device) == fb->fd)
+		return 0;
+
+	if (fb->fd != device->drm.fd) {
+		weston_log("fb was not allocated for scanout device %s\n",
+			   device->drm.filename);
+		return -1;
+	}
+
+	for (plane = 0; plane < gbm_bo_get_plane_count(fb->bo); plane++) {
+		ret = drm_fb_import_plane(device, fb, plane);
+		if (ret)
+			goto err;
+	}
+
+	fb->scanout_device = device;
+
+	return 0;
+err:
+	for (; plane >= 0; plane--) {
+		gem_handle_put(device, fb->handles[plane]);
+		fb->handles[plane] = 0;
+	}
+
+	return ret;
+#endif
+}
+
+static int
+drm_fb_addfb(struct drm_device *device, struct drm_fb *fb)
 {
 	int ret = -EINVAL;
 	uint64_t mods[4] = { };
 	size_t i;
 
+	ret = drm_fb_maybe_import(device, fb);
+	if (ret)
+		return ret;
+
 	/* If we have a modifier set, we must only use the WithModifiers
 	 * entrypoint; we cannot import it through legacy ioctls. */
-	if (b->fb_modifiers && fb->modifier != DRM_FORMAT_MOD_INVALID) {
+	if (device->fb_modifiers && fb->modifier != DRM_FORMAT_MOD_INVALID) {
 		/* KMS demands that if a modifier is set, it must be the same
 		 * for all planes. */
 		for (i = 0; i < ARRAY_LENGTH(mods) && fb->handles[i]; i++)
@@ -98,7 +240,7 @@ drm_fb_addfb(struct drm_backend *b, struct drm_fb *fb)
 
 	/* Legacy AddFB can't always infer the format from depth/bpp alone, so
 	 * check if our format is one of the lucky ones. */
-	if (!fb->format->depth || !fb->format->bpp)
+	if (!fb->format->addfb_legacy_depth || !fb->format->bpp)
 		return ret;
 
 	/* Cannot fall back to AddFB for multi-planar formats either. */
@@ -106,13 +248,13 @@ drm_fb_addfb(struct drm_backend *b, struct drm_fb *fb)
 		return ret;
 
 	ret = drmModeAddFB(fb->fd, fb->width, fb->height,
-			   fb->format->depth, fb->format->bpp,
+			   fb->format->addfb_legacy_depth, fb->format->bpp,
 			   fb->strides[0], fb->handles[0], &fb->fb_id);
 	return ret;
 }
 
 struct drm_fb *
-drm_fb_create_dumb(struct drm_backend *b, int width, int height,
+drm_fb_create_dumb(struct drm_device *device, int width, int height,
 		   uint32_t format)
 {
 	struct drm_fb *fb;
@@ -127,6 +269,8 @@ drm_fb_create_dumb(struct drm_backend *b, int width, int height,
 		return NULL;
 	fb->refcnt = 1;
 
+	fb->backend = device->backend;
+
 	fb->format = pixel_format_get_info(format);
 	if (!fb->format) {
 		weston_log("failed to look up format 0x%lx\n",
@@ -134,7 +278,7 @@ drm_fb_create_dumb(struct drm_backend *b, int width, int height,
 		goto err_fb;
 	}
 
-	if (!fb->format->depth || !fb->format->bpp) {
+	if (!fb->format->addfb_legacy_depth || !fb->format->bpp) {
 		weston_log("format 0x%lx is not compatible with dumb buffers\n",
 			   (unsigned long) format);
 		goto err_fb;
@@ -145,7 +289,7 @@ drm_fb_create_dumb(struct drm_backend *b, int width, int height,
 	create_arg.width = width;
 	create_arg.height = height;
 
-	ret = drmIoctl(b->drm.fd, DRM_IOCTL_MODE_CREATE_DUMB, &create_arg);
+	ret = drmIoctl(device->drm.fd, DRM_IOCTL_MODE_CREATE_DUMB, &create_arg);
 	if (ret)
 		goto err_fb;
 
@@ -157,9 +301,9 @@ drm_fb_create_dumb(struct drm_backend *b, int width, int height,
 	fb->size = create_arg.size;
 	fb->width = width;
 	fb->height = height;
-	fb->fd = b->drm.fd;
+	fb->fd = device->drm.fd;
 
-	if (drm_fb_addfb(b, fb) != 0) {
+	if (drm_fb_addfb(device, fb) != 0) {
 		weston_log("failed to create kms fb: %s\n", strerror(errno));
 		goto err_bo;
 	}
@@ -171,18 +315,18 @@ drm_fb_create_dumb(struct drm_backend *b, int width, int height,
 		goto err_add_fb;
 
 	fb->map = mmap(NULL, fb->size, PROT_WRITE,
-		       MAP_SHARED, b->drm.fd, map_arg.offset);
+		       MAP_SHARED, device->drm.fd, map_arg.offset);
 	if (fb->map == MAP_FAILED)
 		goto err_add_fb;
 
 	return fb;
 
 err_add_fb:
-	drmModeRmFB(b->drm.fd, fb->fb_id);
+	drmModeRmFB(device->drm.fd, fb->fb_id);
 err_bo:
 	memset(&destroy_arg, 0, sizeof(destroy_arg));
 	destroy_arg.handle = create_arg.handle;
-	drmIoctl(b->drm.fd, DRM_IOCTL_MODE_DESTROY_DUMB, &destroy_arg);
+	drmIoctl(device->drm.fd, DRM_IOCTL_MODE_DESTROY_DUMB, &destroy_arg);
 err_fb:
 	free(fb);
 	return NULL;
@@ -209,24 +353,30 @@ drm_fb_destroy_gbm(struct gbm_bo *bo, void *data)
 static void
 drm_fb_destroy_dmabuf(struct drm_fb *fb)
 {
+	int i;
+
 	/* We deliberately do not close the GEM handles here; GBM manages
 	 * their lifetime through the BO. */
 	if (fb->bo)
 		gbm_bo_destroy(fb->bo);
+
+	/*
+	 * If we imported the dmabuf into a scanout device, we are responsible
+	 * for closing the GEM handle.
+	 */
+	for (i = 0; i < 4; i++)
+		if (fb->scanout_device && fb->handles[i] != 0)
+			gem_handle_put(fb->scanout_device, fb->handles[i]);
+
 	drm_fb_destroy(fb);
 }
 
 static struct drm_fb *
 drm_fb_get_from_dmabuf(struct linux_dmabuf_buffer *dmabuf,
-		       struct drm_backend *backend, bool is_opaque,
+		       struct drm_device *device, bool is_opaque,
 		       uint32_t *try_view_on_plane_failure_reasons)
 {
-#ifndef HAVE_GBM_FD_IMPORT
-	/* Importing a buffer to KMS requires explicit modifiers, so
-	 * we can't continue with the legacy GBM_BO_IMPORT_FD instead
-	 * of GBM_BO_IMPORT_FD_MODIFIER. */
-	return NULL;
-#else
+	struct drm_backend *backend = device->backend;
 	struct drm_fb *fb;
 	int i;
 	struct gbm_import_fd_modifier_data import_mod = {
@@ -269,6 +419,7 @@ drm_fb_get_from_dmabuf(struct linux_dmabuf_buffer *dmabuf,
 
 	fb->refcnt = 1;
 	fb->type = BUFFER_DMABUF;
+	fb->backend = device->backend;
 
 	ARRAY_COPY(import_mod.fds, dmabuf->attributes.fd);
 	ARRAY_COPY(import_mod.strides, dmabuf->attributes.stride);
@@ -276,14 +427,18 @@ drm_fb_get_from_dmabuf(struct linux_dmabuf_buffer *dmabuf,
 
 	fb->bo = gbm_bo_import(backend->gbm, GBM_BO_IMPORT_FD_MODIFIER,
 			       &import_mod, GBM_BO_USE_SCANOUT);
-	if (!fb->bo)
+	if (!fb->bo) {
+		if (try_view_on_plane_failure_reasons)
+			*try_view_on_plane_failure_reasons |=
+				FAILURE_REASONS_GBM_BO_IMPORT_FAILED;
 		goto err_free;
+	}
 
 	fb->width = dmabuf->attributes.width;
 	fb->height = dmabuf->attributes.height;
 	fb->modifier = dmabuf->attributes.modifier[0];
 	fb->size = 0;
-	fb->fd = backend->drm.fd;
+	fb->fd = device->drm.fd;
 
 	ARRAY_COPY(fb->strides, dmabuf->attributes.stride);
 	ARRAY_COPY(fb->offsets, dmabuf->attributes.offset);
@@ -298,10 +453,10 @@ drm_fb_get_from_dmabuf(struct linux_dmabuf_buffer *dmabuf,
 	if (is_opaque)
 		fb->format = pixel_format_get_opaque_substitute(fb->format);
 
-	if (backend->min_width > fb->width ||
-	    fb->width > backend->max_width ||
-	    backend->min_height > fb->height ||
-	    fb->height > backend->max_height) {
+	if (device->min_width > fb->width ||
+	    fb->width > device->max_width ||
+	    device->min_height > fb->height ||
+	    fb->height > device->max_height) {
 		weston_log("bo geometry out of bounds\n");
 		goto err_free;
 	}
@@ -311,12 +466,15 @@ drm_fb_get_from_dmabuf(struct linux_dmabuf_buffer *dmabuf,
 		union gbm_bo_handle handle;
 
 	        handle = gbm_bo_get_handle_for_plane(fb->bo, i);
-		if (handle.s32 == -1)
+		if (handle.s32 == -1) {
+			*try_view_on_plane_failure_reasons |=
+				FAILURE_REASONS_GBM_BO_GET_HANDLE_FAILED;
 			goto err_free;
+		}
 		fb->handles[i] = handle.u32;
 	}
 
-	if (drm_fb_addfb(backend, fb) != 0) {
+	if (drm_fb_addfb(device, fb) != 0) {
 		if (try_view_on_plane_failure_reasons)
 			*try_view_on_plane_failure_reasons |=
 				FAILURE_REASONS_ADD_FB_FAILED;
@@ -328,17 +486,14 @@ drm_fb_get_from_dmabuf(struct linux_dmabuf_buffer *dmabuf,
 err_free:
 	drm_fb_destroy_dmabuf(fb);
 	return NULL;
-#endif
 }
 
 struct drm_fb *
-drm_fb_get_from_bo(struct gbm_bo *bo, struct drm_backend *backend,
+drm_fb_get_from_bo(struct gbm_bo *bo, struct drm_device *device,
 		   bool is_opaque, enum drm_fb_type type)
 {
 	struct drm_fb *fb = gbm_bo_get_user_data(bo);
-#ifdef HAVE_GBM_MODIFIERS
 	int i;
-#endif
 
 	if (fb) {
 		assert(fb->type == type);
@@ -351,15 +506,15 @@ drm_fb_get_from_bo(struct gbm_bo *bo, struct drm_backend *backend,
 
 	fb->type = type;
 	fb->refcnt = 1;
+	fb->backend = device->backend;
 	fb->bo = bo;
-	fb->fd = backend->drm.fd;
+	fb->fd = device->drm.fd;
 
 	fb->width = gbm_bo_get_width(bo);
 	fb->height = gbm_bo_get_height(bo);
 	fb->format = pixel_format_get_info(gbm_bo_get_format(bo));
 	fb->size = 0;
 
-#ifdef HAVE_GBM_MODIFIERS
 	fb->modifier = gbm_bo_get_modifier(bo);
 	fb->num_planes = gbm_bo_get_plane_count(bo);
 	for (i = 0; i < fb->num_planes; i++) {
@@ -367,12 +522,6 @@ drm_fb_get_from_bo(struct gbm_bo *bo, struct drm_backend *backend,
 		fb->handles[i] = gbm_bo_get_handle_for_plane(bo, i).u32;
 		fb->offsets[i] = gbm_bo_get_offset(bo, i);
 	}
-#else
-	fb->num_planes = 1;
-	fb->strides[0] = gbm_bo_get_stride(bo);
-	fb->handles[0] = gbm_bo_get_handle(bo).u32;
-	fb->modifier = DRM_FORMAT_MOD_INVALID;
-#endif
 
 	if (!fb->format) {
 		weston_log("couldn't look up format 0x%lx\n",
@@ -385,15 +534,15 @@ drm_fb_get_from_bo(struct gbm_bo *bo, struct drm_backend *backend,
 	if (is_opaque)
 		fb->format = pixel_format_get_opaque_substitute(fb->format);
 
-	if (backend->min_width > fb->width ||
-	    fb->width > backend->max_width ||
-	    backend->min_height > fb->height ||
-	    fb->height > backend->max_height) {
+	if (device->min_width > fb->width ||
+	    fb->width > device->max_width ||
+	    device->min_height > fb->height ||
+	    fb->height > device->max_height) {
 		weston_log("bo geometry out of bounds\n");
 		goto err_free;
 	}
 
-	if (drm_fb_addfb(backend, fb) != 0) {
+	if (drm_fb_addfb(device, fb) != 0) {
 		if (type == BUFFER_GBM_SURFACE)
 			weston_log("failed to create kms fb: %s\n",
 				   strerror(errno));
@@ -444,27 +593,30 @@ drm_fb_unref(struct drm_fb *fb)
 
 #ifdef BUILD_DRM_GBM
 bool
-drm_can_scanout_dmabuf(struct weston_compositor *ec,
+drm_can_scanout_dmabuf(struct weston_backend *backend,
 		       struct linux_dmabuf_buffer *dmabuf)
 {
+	struct drm_backend *b = container_of(backend, struct drm_backend, base);
 	struct drm_fb *fb;
-	struct drm_backend *b = to_drm_backend(ec);
+	struct drm_device *device = b->drm;
 	bool ret = false;
+	uint32_t try_reason = 0x0;
 
-	fb = drm_fb_get_from_dmabuf(dmabuf, b, true, NULL);
+	fb = drm_fb_get_from_dmabuf(dmabuf, device, true, &try_reason);
 	if (fb)
 		ret = true;
 
 	drm_fb_unref(fb);
-	drm_debug(b, "[dmabuf] dmabuf %p, import test %s\n", dmabuf,
-		      ret ? "succeeded" : "failed");
+	drm_debug(b, "[dmabuf] dmabuf %p, import test %s, with reason 0x%x\n", dmabuf,
+		      ret ? "succeeded" : "failed", try_reason);
 	return ret;
 }
 
 static bool
 drm_fb_compatible_with_plane(struct drm_fb *fb, struct drm_plane *plane)
 {
-	struct drm_backend *b = plane->backend;
+	struct drm_device *device = plane->device;
+	struct drm_backend *b = device->backend;
 	struct weston_drm_format *fmt;
 
 	/* Check whether the format is supported */
@@ -501,69 +653,86 @@ drm_fb_compatible_with_plane(struct drm_fb *fb, struct drm_plane *plane)
 static void
 drm_fb_handle_buffer_destroy(struct wl_listener *listener, void *data)
 {
-	struct drm_buffer_fb *buf_fb =
-		container_of(listener, struct drm_buffer_fb, buffer_destroy_listener);
+	struct drm_fb_private *private =
+		container_of(listener, struct drm_fb_private, buffer_destroy_listener);
+	struct drm_buffer_fb *buf_fb;
+	struct drm_buffer_fb *tmp;
 
-	if (buf_fb->fb) {
-		assert(buf_fb->fb->type == BUFFER_CLIENT ||
-		       buf_fb->fb->type == BUFFER_DMABUF);
-		drm_fb_unref(buf_fb->fb);
+	wl_list_remove(&private->buffer_destroy_listener.link);
+
+	wl_list_for_each_safe(buf_fb, tmp, &private->buffer_fb_list, link) {
+		if (buf_fb->fb) {
+			assert(buf_fb->fb->type == BUFFER_CLIENT ||
+			       buf_fb->fb->type == BUFFER_DMABUF);
+			drm_fb_unref(buf_fb->fb);
+		}
+		wl_list_remove(&buf_fb->link);
+		free(buf_fb);
 	}
 
-	free(buf_fb);
+	free(private);
 }
 
 struct drm_fb *
-drm_fb_get_from_view(struct drm_output_state *state, struct weston_view *ev,
-		     uint32_t *try_view_on_plane_failure_reasons)
+drm_fb_get_from_paint_node(struct drm_output_state *state,
+			   struct weston_paint_node *pnode)
 {
 	struct drm_output *output = state->output;
-	struct drm_backend *b = to_drm_backend(output->base.compositor);
+	struct drm_backend *b = output->backend;
+	struct drm_device *device = output->device;
+	struct weston_view *ev = pnode->view;
 	struct weston_buffer *buffer = ev->surface->buffer_ref.buffer;
+	struct drm_fb_private *private;
 	struct drm_buffer_fb *buf_fb;
 	bool is_opaque = weston_view_is_opaque(ev, &ev->transform.boundingbox);
-	struct linux_dmabuf_buffer *dmabuf;
 	struct drm_fb *fb;
 	struct drm_plane *plane;
 
-	if (ev->alpha != 1.0f)
-		return NULL;
-
-	if (!drm_view_transform_supported(ev, &output->base))
-		return NULL;
-
 	if (ev->surface->protection_mode == WESTON_SURFACE_PROTECTION_MODE_ENFORCED &&
-	    ev->surface->desired_protection > output->base.current_protection)
+	    ev->surface->desired_protection > output->base.current_protection) {
+		pnode->try_view_on_plane_failure_reasons |=
+			FAILURE_REASONS_INADEQUATE_CONTENT_PROTECTION;
 		return NULL;
+	}
 
-	if (!buffer)
+	if (!buffer) {
+		pnode->try_view_on_plane_failure_reasons |= FAILURE_REASONS_NO_BUFFER;
 		return NULL;
+	}
 
-	if (buffer->backend_private) {
-		buf_fb = buffer->backend_private;
-		*try_view_on_plane_failure_reasons |= buf_fb->failure_reasons;
-		return buf_fb->fb ? drm_fb_ref(buf_fb->fb) : NULL;
+	if (!buffer->backend_private) {
+		private = zalloc(sizeof(*private));
+		buffer->backend_private = private;
+		wl_list_init(&private->buffer_fb_list);
+		private->buffer_destroy_listener.notify = drm_fb_handle_buffer_destroy;
+		wl_signal_add(&buffer->destroy_signal, &private->buffer_destroy_listener);
+	} else {
+		private = buffer->backend_private;
+	}
+
+	wl_list_for_each(buf_fb, &private->buffer_fb_list, link) {
+		if (buf_fb->device == device) {
+			pnode->try_view_on_plane_failure_reasons |= buf_fb->failure_reasons;
+			return buf_fb->fb ? drm_fb_ref(buf_fb->fb) : NULL;
+		}
 	}
 
 	buf_fb = zalloc(sizeof(*buf_fb));
-	buffer->backend_private = buf_fb;
-	buf_fb->buffer_destroy_listener.notify = drm_fb_handle_buffer_destroy;
-	wl_signal_add(&buffer->destroy_signal, &buf_fb->buffer_destroy_listener);
-
-	if (wl_shm_buffer_get(buffer->resource))
-		goto unsuitable;
+	buf_fb->device = device;
+	wl_list_insert(&private->buffer_fb_list, &buf_fb->link);
 
 	/* GBM is used for dmabuf import as well as from client wl_buffer. */
-	if (!b->gbm)
+	if (!b->gbm) {
+		pnode->try_view_on_plane_failure_reasons |= FAILURE_REASONS_NO_GBM;
 		goto unsuitable;
+	}
 
-	dmabuf = linux_dmabuf_buffer_get(buffer->resource);
-	if (dmabuf) {
-		fb = drm_fb_get_from_dmabuf(dmabuf, b, is_opaque,
+	if (buffer->type == WESTON_BUFFER_DMABUF) {
+		fb = drm_fb_get_from_dmabuf(buffer->dmabuf, device, is_opaque,
 					    &buf_fb->failure_reasons);
 		if (!fb)
 			goto unsuitable;
-	} else {
+	} else if (buffer->type == WESTON_BUFFER_RENDERER_OPAQUE) {
 		struct gbm_bo *bo;
 
 		bo = gbm_bo_import(b->gbm, GBM_BO_IMPORT_WL_BUFFER,
@@ -571,18 +740,27 @@ drm_fb_get_from_view(struct drm_output_state *state, struct weston_view *ev,
 		if (!bo)
 			goto unsuitable;
 
-		fb = drm_fb_get_from_bo(bo, b, is_opaque, BUFFER_CLIENT);
+		fb = drm_fb_get_from_bo(bo, device, is_opaque, BUFFER_CLIENT);
 		if (!fb) {
+			pnode->try_view_on_plane_failure_reasons |=
+				(1 << FAILURE_REASONS_ADD_FB_FAILED);
 			gbm_bo_destroy(bo);
 			goto unsuitable;
 		}
+	} else {
+		pnode->try_view_on_plane_failure_reasons |= FAILURE_REASONS_BUFFER_TYPE;
+		goto unsuitable;
 	}
 
 	/* Check if this buffer can ever go on any planes. If it can't, we have
 	 * no reason to ever have a drm_fb, so we fail it here. */
-	wl_list_for_each(plane, &b->plane_list, link) {
+	wl_list_for_each(plane, &device->plane_list, link) {
+		/* only SHM buffers can go into cursor planes */
+		if (plane->type == WDRM_PLANE_TYPE_CURSOR)
+			continue;
+
 		if (drm_fb_compatible_with_plane(fb, plane))
-			fb->plane_mask |= (1 << plane->plane_idx);
+			fb->plane_mask |= 1 << (plane->plane_idx);
 	}
 	if (fb->plane_mask == 0) {
 		drm_fb_unref(fb);
@@ -599,7 +777,7 @@ drm_fb_get_from_view(struct drm_output_state *state, struct weston_view *ev,
 	return fb;
 
 unsuitable:
-	*try_view_on_plane_failure_reasons |= buf_fb->failure_reasons;
+	pnode->try_view_on_plane_failure_reasons |= buf_fb->failure_reasons;
 	return NULL;
 }
 #endif
