@@ -415,7 +415,7 @@ drag_surface_configure(struct weston_drag *drag,
 		       struct weston_coord_surface new_origin)
 {
 	struct weston_layer_entry *list;
-	float fx, fy;
+	struct weston_coord_global pos;
 
 	assert((pointer != NULL && touch == NULL) ||
 			(pointer == NULL && touch != NULL));
@@ -435,20 +435,16 @@ drag_surface_configure(struct weston_drag *drag,
 		drag->icon->is_mapped = true;
 	}
 
-	assert(drag->offset.coordinate_space_id &&
-	       drag->offset.coordinate_space_id == new_origin.coordinate_space_id);
-	drag->offset.c = weston_coord_add(drag->offset.c, new_origin.c);
+	drag->offset = weston_coord_surface_add(drag->offset, new_origin);
 
 	/* init to 0 for avoiding a compile warning */
-	fx = fy = 0;
-	if (pointer) {
-		fx = pointer->pos.c.x + drag->offset.c.x;
-		fy = pointer->pos.c.y + drag->offset.c.y;
-	} else if (touch) {
-		fx = wl_fixed_to_double(touch->grab_x) + drag->offset.c.x;
-		fy = wl_fixed_to_double(touch->grab_y) + drag->offset.c.y;
-	}
-	weston_view_set_position(drag->icon, fx, fy);
+	pos.c = weston_coord(0, 0);
+	if (pointer)
+		pos = pointer->pos;
+	else if (touch)
+		pos = touch->grab_pos;
+
+	weston_view_set_position_with_offset(drag->icon, pos, drag->offset);
 }
 
 static int
@@ -617,16 +613,14 @@ drag_grab_motion(struct weston_pointer_grab *grab,
 	struct weston_pointer_drag *drag =
 		container_of(grab, struct weston_pointer_drag, grab);
 	struct weston_pointer *pointer = drag->grab.pointer;
-	float fx, fy;
 	uint32_t msecs;
 
 	weston_pointer_move(pointer, event);
 
 	if (drag->base.icon) {
-		fx = pointer->pos.c.x + drag->base.offset.c.x;
-		fy = pointer->pos.c.y + drag->base.offset.c.y;
-		weston_view_set_position(drag->base.icon, fx, fy);
-		weston_view_schedule_repaint(drag->base.icon);
+		weston_view_set_position_with_offset(drag->base.icon,
+						     pointer->pos,
+						     drag->base.offset);
 	}
 
 	if (drag->base.focus_resource) {
@@ -757,7 +751,7 @@ static const struct weston_pointer_grab_interface pointer_drag_grab_interface =
 static void
 drag_grab_touch_down(struct weston_touch_grab *grab,
 		     const struct timespec *time, int touch_id,
-		     wl_fixed_t sx, wl_fixed_t sy)
+		     struct weston_coord_global c)
 {
 }
 
@@ -798,46 +792,39 @@ static void
 drag_grab_touch_focus(struct weston_touch_drag *drag)
 {
 	struct weston_touch *touch = drag->grab.touch;
-	struct weston_coord_global pos;
 
-	pos.c = weston_coord_from_fixed(touch->grab_x, touch->grab_y);
-	drag_grab_focus_internal(&drag->base, touch->seat, pos);
+	drag_grab_focus_internal(&drag->base, touch->seat, touch->grab_pos);
 }
 
 static void
 drag_grab_touch_motion(struct weston_touch_grab *grab,
 		       const struct timespec *time,
-		       int touch_id, wl_fixed_t x, wl_fixed_t y)
+		       int touch_id, struct weston_coord_global unused)
 {
 	struct weston_touch_drag *touch_drag =
 		container_of(grab, struct weston_touch_drag, grab);
 	struct weston_touch *touch = grab->touch;
-	float fx, fy;
 	uint32_t msecs;
 
 	if (touch_id != touch->grab_touch_id)
 		return;
 
 	drag_grab_touch_focus(touch_drag);
-	if (touch_drag->base.icon) {
-		fx = wl_fixed_to_double(touch->grab_x) +
-		     touch_drag->base.offset.c.x;
-		fy = wl_fixed_to_double(touch->grab_y) +
-		     touch_drag->base.offset.c.y;
-		weston_view_set_position(touch_drag->base.icon, fx, fy);
-		weston_view_schedule_repaint(touch_drag->base.icon);
-	}
+	if (touch_drag->base.icon)
+		weston_view_set_position_with_offset(touch_drag->base.icon,
+						     touch->grab_pos,
+						     touch_drag->base.offset);
 
 	if (touch_drag->base.focus_resource) {
-		struct weston_coord_global tmp_g;
-		struct weston_coord_surface c;
+		struct weston_coord_surface surf_pos;
 
-		tmp_g.c = weston_coord_from_fixed(touch->grab_x, touch->grab_y);
 		msecs = timespec_to_msec(time);
-		c = weston_coord_global_to_surface(touch_drag->base.focus, tmp_g);
+		surf_pos = weston_coord_global_to_surface(touch_drag->base.focus,
+							  touch->grab_pos);
 		wl_data_device_send_motion(touch_drag->base.focus_resource,
-					   msecs, wl_fixed_from_double(c.c.x),
-					   wl_fixed_from_double(c.c.y));
+					   msecs,
+					   wl_fixed_from_double(surf_pos.c.x),
+					   wl_fixed_from_double(surf_pos.c.y));
 	}
 }
 
