@@ -1,23 +1,26 @@
 /*
  * Copyright © 2008 Kristian Høgsberg
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #ifndef _CAIRO_UTIL_H
@@ -25,13 +28,19 @@
 
 #include <stdint.h>
 #include <cairo.h>
+#ifdef HAVE_PANGO
+#include <pango/pangocairo.h>
+#endif
+
+#include <wayland-client.h>
+#include <wayland-util.h>
 
 void
 surface_flush_device(cairo_surface_t *surface);
 
 void
-tile_mask(cairo_t *cr, cairo_surface_t *surface,
-	  int x, int y, int width, int height, int margin, int top_margin);
+render_shadow(cairo_t *cr, cairo_surface_t *surface,
+	      int x, int y, int width, int height, int margin, int top_margin);
 
 void
 tile_source(cairo_t *cr, cairo_surface_t *surface,
@@ -51,6 +60,9 @@ struct theme {
 	int margin;
 	int width;
 	int titlebar_height;
+#ifdef HAVE_PANGO
+	PangoContext *pango_context;
+#endif
 };
 
 struct theme *
@@ -67,9 +79,10 @@ enum {
 void
 theme_set_background_source(struct theme *t, cairo_t *cr, uint32_t flags);
 void
-theme_render_frame(struct theme *t, 
+theme_render_frame(struct theme *t,
 		   cairo_t *cr, int width, int height,
-		   const char *title, uint32_t flags);
+		   const char *title, cairo_rectangle_int_t *title_rect,
+		   struct wl_list *buttons, uint32_t flags);
 
 enum theme_location {
 	THEME_LOCATION_INTERIOR = 0,
@@ -117,14 +130,9 @@ enum {
 	FRAME_BUTTON_ALL = 0x7
 };
 
-enum frame_button_state {
-	FRAME_BUTTON_RELEASED = 0,
-	FRAME_BUTTON_PRESSED = 1
-};
-
 struct frame *
 frame_create(struct theme *t, int32_t width, int32_t height, uint32_t buttons,
-	     const char *title);
+             const char *title, cairo_surface_t *icon);
 
 void
 frame_destroy(struct frame *frame);
@@ -133,6 +141,10 @@ frame_destroy(struct frame *frame);
 int
 frame_set_title(struct frame *frame, const char *title);
 
+/* May set FRAME_STATUS_REPAINT */
+void
+frame_set_icon(struct frame *frame, cairo_surface_t *icon);
+
 /* May set FRAME_STATUS_REPAINT */
 void
 frame_set_flag(struct frame *frame, enum frame_flag flag);
@@ -155,6 +167,14 @@ frame_width(struct frame *frame);
 int32_t
 frame_height(struct frame *frame);
 
+void
+frame_border_sizes(struct frame *frame, int32_t *top, int32_t *bottom,
+		   int32_t *left, int32_t *right);
+
+void
+frame_decoration_sizes(struct frame *frame, int32_t *top, int32_t *bottom,
+                       int32_t *left, int32_t *right);
+
 void
 frame_interior(struct frame *frame, int32_t *x, int32_t *y,
 	       int32_t *width, int32_t *height);
@@ -165,6 +185,9 @@ void
 frame_opaque_rect(struct frame *frame, int32_t *x, int32_t *y,
 		  int32_t *width, int32_t *height);
 
+int
+frame_get_shadow_margin(struct frame *frame);
+
 uint32_t
 frame_status(struct frame *frame);
 
@@ -199,15 +222,33 @@ frame_pointer_leave(struct frame *frame, void *pointer);
  */
 enum theme_location
 frame_pointer_button(struct frame *frame, void *pointer,
-		     uint32_t button, enum frame_button_state state);
+		     uint32_t button, enum wl_pointer_button_state state);
 
-void
+enum theme_location
 frame_touch_down(struct frame *frame, void *data, int32_t id, int x, int y);
 
 void
 frame_touch_up(struct frame *frame, void *data, int32_t id);
 
+enum theme_location
+frame_double_click(struct frame *frame, void *pointer,
+		   uint32_t button, enum wl_pointer_button_state state);
+
+void
+frame_double_touch_down(struct frame *frame, void *data, int32_t id,
+			int x, int y);
+
+void
+frame_double_touch_up(struct frame *frame, void *data, int32_t id);
+
+/* May set FRAME_STATUS_REPAINT */
+enum theme_location
+frame_tablet_tool_motion(struct frame *frame, void *pointer, int x, int y);
+
 void
 frame_repaint(struct frame *frame, cairo_t *cr);
 
+void
+cleanup_after_cairo(void);
+
 #endif
